package cz.simecek.inspectionmanager;

import cz.simecek.inspectionmanager.model.*;
import cz.simecek.inspectionmanager.repository.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.core.annotation.Order;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Component;

import java.sql.Date;
import java.util.*;

@Component
@Order(0)
public class ApplicationStartListener implements ApplicationListener<ApplicationReadyEvent> {

    private SystemUserRepository systemUserRepository;
    private RoleRepository roleRepository;
    private final BCryptPasswordEncoder bCryptPasswordEncoder;
    private CustomerRepository customerRepository;
    private MachineRepository machineRepository;
    private SparePartRepository sparePartRepository;

    @Autowired
    public ApplicationStartListener(SystemUserRepository systemUserRepository, RoleRepository roleRepository, BCryptPasswordEncoder bCryptPasswordEncoder, CustomerRepository customerRepository, MachineRepository machineRepository, SparePartRepository sparePartRepository) {
        this.systemUserRepository = systemUserRepository;
        this.roleRepository = roleRepository;
        this.bCryptPasswordEncoder = bCryptPasswordEncoder;
        this.customerRepository = customerRepository;
        this.machineRepository = machineRepository;
        this.sparePartRepository = sparePartRepository;
    }

    Logger logger = LoggerFactory.getLogger(ApplicationStartListener.class);

    @Override
    public void onApplicationEvent(ApplicationReadyEvent event) {

        List<String> defaultUsers = Arrays.asList("ADMIN", "MANAGER", "EMPLOYEE");
        defaultUsers.forEach(this::createDefaultUser);
    }

    private void createDefaultUser(String name) {
        Role role = new Role();
        role.setName(name);
        Role createdRole = roleRepository.save(role);
        loggInfo("role", name);
        createUser(createdRole, name.toLowerCase());
    }

    private void createUser(Role role, String name) {
        SystemUser user = new SystemUser();
        user.setUserName(name);
        user.setPassword(bCryptPasswordEncoder.encode(name));
        user.setRole(role);
        user.setBlocked(false);
        SystemUser createdUser = systemUserRepository.save(user);
        createTestData(createdUser);
        loggInfo("user", name);
    }

    private void loggInfo(String action, String name) {
        String message = String.format("Created %s : %s", action, name);
        logger.info(message);
    }

    private void createTestData(SystemUser user) {
        Customer customer = new Customer();
        customer.setFirstName("Jan");
        customer.setLastName("Novák");
        customer.setCity("Novákov");
        customer.setPhone("123456789");
        customer.setEmail("jan@novak.com");
        customer.setFocus("Tetovací uživatel");
        customer.setCreatedAt(new Date(System.currentTimeMillis()));
        customer.setDeleted(false);
        customer.setCreatedBy(user);
        Customer createdCustomer = customerRepository.save(customer);
        Customer deleteCustomer = new Customer();
        deleteCustomer.setFirstName("Jan delete");
        deleteCustomer.setLastName("Novák delete");
        deleteCustomer.setCity("Novákov");
        deleteCustomer.setPhone("123456789");
        deleteCustomer.setEmail("jan@novak.com");
        deleteCustomer.setFocus("Tetovací uživatel");
        deleteCustomer.setCreatedAt(new Date(System.currentTimeMillis()));
        deleteCustomer.setDeleted(false);
        deleteCustomer.setCreatedBy(user);
        customerRepository.save(deleteCustomer);
        Machine machine = new Machine();
        machine.setName("HUSQWARNA 350");
        machine.setCode(String.format("GBNA99212%s", user.getUserName()));
        machine.setNextInspectionAt(new Date(System.currentTimeMillis()));
        machine.setNote("Testovací stroj");
        machine.setIsBoughtLocally(true);
        machine.setBoughtAt(new Date(System.currentTimeMillis()));
        machine.setIsWorking(true);
        machine.setDeleted(false);
        machine.setCreatedAt(new Date(System.currentTimeMillis()));
        machine.setCustomer(createdCustomer);
        machine.setCreatedBy(user);
        Machine createdMachine = machineRepository.save(machine);
        SparePart sparePart = new SparePart();
        sparePart.setName("Motor");
        sparePart.setCode(String.format("BNB22%s", user.getUserName()));
        sparePart.setPrice("20 000");
        createdMachine.getSpareParts().add(sparePart);
        Category category = new Category();
        category.setName("MACHINES");
        category.setCreatedAt(new Date(System.currentTimeMillis()));
        category.setCreatedBy(user);
        createdMachine.getCategories().add(category);
        machineRepository.save(createdMachine);
    }
}
