package cz.simecek.inspectionmanager.controller;

import cz.simecek.inspectionmanager.dto.CategoryDTO;
import cz.simecek.inspectionmanager.service.category.CategoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.validation.Valid;

@Controller
public class CategoryController {

    private final CategoryService categoryService;

    private static final String REDIRECT_TO_INDEX = "redirect:/all/category";
    private static final String INDEX_TEMPLATE = "/category/list";
    private static final String CREATE_TEMPLATE = "/category/new";
    private static final String DETAIL_TEMPLATE = "/category/detail";
    private static final String UPDATE_TEMPLATE = "/category/update";

    private static final String CATEGORIES = "categories";
    private static final String CATEGORY_DTO = "categoryDTO";
    private static final String MACHINES = "machines";
    private static final String SUCCESS_MESSAGE = "successMessage";

    @Autowired
    public CategoryController(CategoryService categoryService) {
        this.categoryService = categoryService;
    }

    @GetMapping("/all/category")
    public String index(Model model) {
        model.addAttribute(CATEGORIES, categoryService.getAll());
        return INDEX_TEMPLATE;
    }

    @PostMapping("/update/category/{categoryId}/add/machine/{machineId}")
    public String addMachine(@PathVariable("categoryId") Long categoryId, @PathVariable("machineId") Long machineId, RedirectAttributes redirectAttributes) {
        categoryService.saveCategoryToMachine(categoryId, machineId);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Kategorie byla úspěšně aktualizována.");
        return REDIRECT_TO_INDEX;
    }

    @GetMapping("/new/category")
    public String createForm(CategoryDTO categoryDTO) {
        return CREATE_TEMPLATE;
    }

    @PostMapping("/new/category")
    public String create(@Valid CategoryDTO categoryDTO, BindingResult result, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            return CREATE_TEMPLATE;
        }
        categoryService.save(categoryDTO);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Kategorie byla úspěšně vytvořena.");
        return REDIRECT_TO_INDEX;
    }

    @GetMapping("/detail/category/{id}")
    public String detail(@PathVariable("id") Long categoryId, Model model) {
        CategoryDTO categoryDTO = categoryService.getById(categoryId);
        model.addAttribute(CATEGORY_DTO, categoryDTO);
        model.addAttribute(MACHINES, categoryService.getAllMachinesExcludeByCategoryId(categoryId));
        return DETAIL_TEMPLATE;
    }

    @GetMapping("/update/category/{id}")
    public String updateForm(@PathVariable("id") Long categoryId, Model model) {
        CategoryDTO categoryDTO = categoryService.getById(categoryId);
        model.addAttribute(CATEGORY_DTO, categoryDTO);
        return UPDATE_TEMPLATE;
    }

    @PostMapping("/update/category/{id}")
    public String update(@PathVariable("id") Long categoryId, @Valid CategoryDTO categoryDTO, BindingResult result, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            categoryDTO.setId(categoryId);
            return UPDATE_TEMPLATE;
        }
        categoryService.update(categoryDTO, categoryId);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Kategorie byla úspěšně aktualizována.");
        return REDIRECT_TO_INDEX;
    }

    @PostMapping("/delete/category/{id}")
    public String delete(@PathVariable("id") Long categoryId, RedirectAttributes redirectAttributes) {
        categoryService.delete(categoryId);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Kategorie byla úspěšně smazána.");
        return REDIRECT_TO_INDEX;
    }
}
