package cz.simecek.inspectionmanager.controller;

import cz.simecek.inspectionmanager.dto.CustomerDTO;
import cz.simecek.inspectionmanager.service.customer.CustomerService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.validation.Valid;

@Controller
public class CustomerController {

    private final CustomerService customerService;

    private static final String REDIRECT_TO_INDEX = "redirect:/all/customer";
    private static final String INDEX_TEMPLATE = "/customer/list";
    private static final String CREATE_TEMPLATE = "/customer/new";
    private static final String DETAIL_TEMPLATE = "/customer/detail";
    private static final String UPDATE_TEMPLATE = "/customer/update";
    private static final String HISTORY_TEMPLATE = "/history/history";
    private static final String RENEW_TEMPLATE = "/customer/renew";
    private static final String REDIRECT_TO_DELETED = "redirect:/renew/customer";

    private static final String CUSTOMERS = "customers";
    private static final String CUSTOMER_DTO = "customerDTO";
    private static final String HISTORIES = "histories";
    private static final String HEADER = "header";
    private static final String SUCCESS_MESSAGE = "successMessage";

    @Autowired
    public CustomerController(CustomerService customerService) {
        this.customerService = customerService;
    }

    @GetMapping("/all/customer")
    public String index(Model model) {
        model.addAttribute(CUSTOMERS, customerService.getAll());
        return INDEX_TEMPLATE;
    }

    @GetMapping("/renew/customer")
    public String getDeleted(Model model) {
        model.addAttribute(CUSTOMERS, customerService.getDeleted());
        return RENEW_TEMPLATE;
    }

    @PostMapping("/renew/customer/{id}")
    public String renewCustomer(@PathVariable Long id, Model model, RedirectAttributes redirectAttributes) {
        customerService.renew(id);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Zákazník byl úspěšně obnoven.");
        return REDIRECT_TO_DELETED;
    }

    @GetMapping("/new/customer")
    public String createForm(CustomerDTO customerDTO) {
        return CREATE_TEMPLATE;
    }

    @PostMapping("/new/customer")
    public String create(@Valid CustomerDTO customerDTO, BindingResult result, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            return CREATE_TEMPLATE;
        }
        customerService.save(customerDTO);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Zákazník byl úspěšně vytvořen.");
        return REDIRECT_TO_INDEX;
    }

    @GetMapping("/detail/customer/{id}")
    public String detail(@PathVariable("id") Long customerId, Model model) {
        model.addAttribute(CUSTOMER_DTO, customerService.getById(customerId));
        return DETAIL_TEMPLATE;
    }

    @GetMapping("/update/customer/{id}")
    public String updateForm(@PathVariable("id") Long customerId, Model model) {
        model.addAttribute(CUSTOMER_DTO, customerService.getById(customerId));
        return UPDATE_TEMPLATE;
    }

    @PostMapping("/update/customer/{id}")
    public String update(@PathVariable("id") Long customerId, @Valid CustomerDTO customerDTO, BindingResult result, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            customerDTO.setId(customerId);
            return UPDATE_TEMPLATE;
        }
        customerService.update(customerDTO, customerId);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Zákazník byl úspěšně aktualizován.");
        return REDIRECT_TO_INDEX;
    }

    @PostMapping("/delete/customer/{id}")
    public String delete(@PathVariable("id") Long customerId, RedirectAttributes redirectAttributes) {
        customerService.delete(customerId);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Zákazník byl úspěšně smazán.");
        return REDIRECT_TO_INDEX;
    }

    @GetMapping("/history/customer/{id}")
    public String history(@PathVariable("id") Long id, Model model) {
        model.addAttribute(HISTORIES, customerService.getHistory(id));
        model.addAttribute(HEADER, "zákazníka");
        return HISTORY_TEMPLATE;
    }
}
