package cz.simecek.inspectionmanager.controller;

import cz.simecek.inspectionmanager.dto.CustomerDTO;
import cz.simecek.inspectionmanager.dto.MachineDTO;
import cz.simecek.inspectionmanager.service.category.CategoryService;
import cz.simecek.inspectionmanager.service.customer.CustomerService;
import cz.simecek.inspectionmanager.service.machine.MachineService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.validation.Valid;

@Controller
public class MachineController {

    private final MachineService machineService;
    private final CategoryService categoryService;
    private final CustomerService customerService;

    private static final String REDIRECT_TO_INDEX = "redirect:/all/machine";
    private static final String CREATE_TEMPLATE = "/machine/new";
    private static final String INDEX_TEMPLATE = "/machine/list";
    private static final String DETAIL_TEMPLATE = "/machine/detail";
    private static final String UPDATE_TEMPLATE = "/machine/update";
    private static final String RENEW_TEMPLATE = "/machine/renew";
    private static final String HISTORY_TEMPLATE = "/history/history";
    private static final String REDIRECT_TO_DELETED = "redirect:/renew/machine";

    private static final String MACHINE_DTO = "machineDTO";
    private static final String MACHINES = "machines";
    private static final String CATEGORIES = "categories";
    private static final String CUSTOMER = "customer";
    private static final String HISTORIES = "histories";
    private static final String HEADER = "header";
    private static final String SUCCESS_MESSAGE = "successMessage";
    private static final String CUSTOMER_ID = "customerId";
    private static final String CATEGORY_ID = "categoryId";

    @Autowired
    public MachineController(MachineService machineService, CategoryService categoryService,
                             CustomerService customerService) {
        this.machineService = machineService;
        this.categoryService = categoryService;
        this.customerService = customerService;
    }

    @GetMapping("/all/machine")
    public String index(Model model) {
        model.addAttribute(MACHINES, machineService.getAll());
        return INDEX_TEMPLATE;
    }

    @GetMapping("/renew/machine")
    public String getDeleted(Model model) {
        model.addAttribute(MACHINES, machineService.getDeleted());
        return RENEW_TEMPLATE;
    }

    @PostMapping("/renew/machine/{id}")
    public String renewCustomer(@PathVariable Long id, RedirectAttributes redirectAttributes) {
        machineService.renew(id);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Stroj byl úspěšně obnoven.");
        return REDIRECT_TO_DELETED;
    }

    @GetMapping("/new/machine")
    public String createForm(@RequestParam(name = CUSTOMER_ID) Long customerId, Model model) {
        model.addAttribute(MACHINE_DTO, new MachineDTO());
        model.addAttribute(CUSTOMER_ID, customerId);
        model.addAttribute(CATEGORIES, categoryService.getAll());
        return CREATE_TEMPLATE;
    }

    @PostMapping("/new/machine")
    public String create(@RequestParam(name = CUSTOMER_ID) Long customerId, @Valid MachineDTO machineDTO,
                         BindingResult result, Model model, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            model.addAttribute(CATEGORIES, categoryService.getAll());
            model.addAttribute(CUSTOMER_ID, customerId);
            return CREATE_TEMPLATE;
        }
        machineService.save(machineDTO, customerId);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Stroj byl úspěšně vytvořen.");
        return REDIRECT_TO_INDEX;
    }

    @GetMapping("/filter/machine")
    public String filter(@RequestParam(name = CUSTOMER_ID) Long customerId, Model model) {
        model.addAttribute(MACHINES, machineService.getByCustomer(customerId));
        model.addAttribute(CUSTOMER_ID, customerId);
        return INDEX_TEMPLATE;
    }

    @GetMapping("/detail/machine/{id}")
    public String detail(@PathVariable("id") Long machineId, Model model) {
        MachineDTO machine = machineService.getById(machineId);
        CustomerDTO customer = customerService.getById(machine.getCustomer().getId());
        model.addAttribute(MACHINE_DTO, machine);
        model.addAttribute(CUSTOMER, customer);
        model.addAttribute(CATEGORIES, categoryService.getAll());
        return DETAIL_TEMPLATE;
    }

    @GetMapping("/filter/machine/category/{id}")
    public String filterCategory(@PathVariable("id") Long id, Model model) {
        model.addAttribute(MACHINES, machineService.getAllByCategory(id));
        return INDEX_TEMPLATE;
    }

    @GetMapping("/update/machine/{id}")
    public String updateForm(@PathVariable("id") Long machineId, Model model) {
        model.addAttribute(MACHINE_DTO, machineService.getById(machineId));
        model.addAttribute(CATEGORIES, categoryService.getAll());
        return UPDATE_TEMPLATE;
    }

    @PostMapping("/update/machine/{id}")
    public String update(@PathVariable("id") Long machineId, @Valid MachineDTO machineDTO,
                         BindingResult result, Model model, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            model.addAttribute(CATEGORIES, categoryService.getAll());
            machineDTO.setId(machineId);
            return UPDATE_TEMPLATE;
        }
        machineService.update(machineDTO, machineId);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Stroj byl úspěšně aktualizován.");
        return REDIRECT_TO_INDEX;
    }

    @PostMapping("/delete/machine/{id}")
    public String delete(@PathVariable("id") Long machineId, RedirectAttributes redirectAttributes) {
        machineService.delete(machineId);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Stroj byl úspěšně smazán.");
        return REDIRECT_TO_INDEX;
    }

    @GetMapping("/late/machine")
    public String findLateInspections(Model model) {
        model.addAttribute(MACHINES, machineService.findLateInspections());
        return INDEX_TEMPLATE;
    }

    @GetMapping("/history/machine/{id}")
    public String getHistory(@PathVariable Long id, Model model) {
        model.addAttribute(HISTORIES, machineService.getHistoryById(id));
        model.addAttribute(HEADER, "stroje");
        return HISTORY_TEMPLATE;
    }
}
