package cz.simecek.inspectionmanager.controller;

import cz.simecek.inspectionmanager.dto.PasswordDTO;
import cz.simecek.inspectionmanager.dto.RoleDTO;
import cz.simecek.inspectionmanager.dto.UserDTO;
import cz.simecek.inspectionmanager.dto.UserPasswordDTO;
import cz.simecek.inspectionmanager.service.role.RoleService;
import cz.simecek.inspectionmanager.service.user.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.validation.Valid;

@Controller
public class UserController {

    private static final String REDIRECT_TO_USER_INDEX = "redirect:/user/all";
    private static final String REDIRECT_TO_REGISTER = "redirect:/register";
    private static final String REDIRECT_TO_ACTIVATE = "redirect:/user/activate";
    private static final String REDIRECT_TO_DEACTIVATE = "redirect:/user/deactivate";
    private static final String REDIRECT_TO_UPDATE_PASSWORD_BY_USER = "redirect:/password";

    private static final String CREATE_TEMPLATE = "/user/new";
    private static final String ACTIVATE_TEMPLATE = "/user/activate";
    private static final String DEACTIVATE_TEMPLATE = "/user/deactivate";
    private static final String INDEX_TEMPLATE = "/user/list";
    private static final String DETAIL_TEMPLATE = "/user/detail";
    private static final String UPDATE_PASSWORD_TEMPLATE = "/user/change_password";
    private static final String UPDATE_PASSWORD_BY_USER_TEMPLATE = "/user/user_password";
    private static final String UPDATE_ROLE_TEMPLATE = "/user/change_role";
    private static final String HISTORY_TEMPLATE = "/history/history";

    private static final String ROLES = "roles";
    private static final String USERS = "users";
    private static final String HISTORIES = "histories";
    private static final String USER_DTO = "userDTO";
    private static final String ROLE_DTO = "roleDTO";
    private static final String SUCCESS_MESSAGE = "successMessage";
    private static final String HEADER = "header";

    private final UserService userService;
    private final RoleService roleService;

    @Autowired
    public UserController(UserService userService, RoleService roleService) {
        this.userService = userService;
        this.roleService = roleService;
    }

    @GetMapping("/user/all")
    public String index(Model model) {
        model.addAttribute(USERS, userService.getAll());
        return INDEX_TEMPLATE;
    }

    @GetMapping("/register")
    public String registerForm(UserDTO userDTO, Model model) {
        model.addAttribute(ROLES, roleService.getAll());
        return CREATE_TEMPLATE;
    }

    @PostMapping("/register")
    public String register(@Valid UserDTO userDTO, BindingResult result, Model model, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            model.addAttribute(ROLES, roleService.getAll());
            return CREATE_TEMPLATE;
        }
        userDTO.setBlocked(true);
        userService.save(userDTO);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Uživatel byl úspěšně vytvořen, požádejte administrátora o aktivaci uživatele.");
        return REDIRECT_TO_REGISTER;
    }

    @GetMapping("/user/activate")
    public String activateUser(Model model) {
        model.addAttribute(USERS, userService.getAllBlocked());
        return ACTIVATE_TEMPLATE;
    }

    @PostMapping("/user/activate/{id}")
    public String activate(@PathVariable Long id, RedirectAttributes redirectAttributes) {
        userService.activate(id);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Uživatel byl úspěšně aktivován.");
        return REDIRECT_TO_ACTIVATE;
    }

    @PostMapping("/user/delete/{id}")
    public String delete(@PathVariable("id") Long id, RedirectAttributes redirectAttributes) {
        userService.delete(id);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Uživatel byl úspěšně smazán.");
        return REDIRECT_TO_USER_INDEX;
    }

    @GetMapping("/user/detail/{id}")
    public String detail(@PathVariable("id") Long id, Model model) {
        model.addAttribute(USER_DTO, userService.findById(id));
        return DETAIL_TEMPLATE;
    }

    @GetMapping("/user/password/{id}")
    public String changePasswordForm(@PathVariable("id") Long id, PasswordDTO passwordDTO) {
        passwordDTO.setUserId(id);
        return UPDATE_PASSWORD_TEMPLATE;
    }

    @GetMapping("/password")
    public String updatePasswordByUserForm(UserPasswordDTO userPasswordDTO) {
        return UPDATE_PASSWORD_BY_USER_TEMPLATE;
    }

    @PostMapping("/password")
    public String updatePasswordByUser(@Valid UserPasswordDTO userPasswordDTO, BindingResult result, RedirectAttributes redirectAttributes) {
        userService.updatePasswordByUser(userPasswordDTO);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Heslo bylo úspěšně aktualizováno.");
        return REDIRECT_TO_UPDATE_PASSWORD_BY_USER;

    }

    @PostMapping("/user/password/{id}")
    public String changePassword(@PathVariable("id") Long id, @Valid PasswordDTO passwordDTO, BindingResult result, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            passwordDTO.setUserId(id);
            return UPDATE_PASSWORD_TEMPLATE;
        }
        passwordDTO.setUserId(id);
        userService.changePassword(passwordDTO);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Heslo bylo úspěšně aktualizováno.");
        return REDIRECT_TO_USER_INDEX;
    }

    @GetMapping("/user/role/{id}")
    public String changeRoleForm(@PathVariable("id") Long userId, Model model) {
        RoleDTO roleDTO = new RoleDTO();
        roleDTO.setUserId(userId);
        model.addAttribute(ROLE_DTO, roleDTO);
        model.addAttribute(ROLES, roleService.getAll());
        return UPDATE_ROLE_TEMPLATE;
    }

    @PostMapping("/user/role/{id}")
    public String changeRole(@PathVariable("id") Long userId, @Valid RoleDTO roleDTO, BindingResult result, Model model, RedirectAttributes redirectAttributes) {
        if (result.hasErrors()) {
            roleDTO.setUserId(userId);
            model.addAttribute(ROLES, roleService.getAll());
            return UPDATE_ROLE_TEMPLATE;
        }
        roleDTO.setUserId(userId);
        userService.changeRole(roleDTO);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Role byla úspěšně aktualizována.");
        return REDIRECT_TO_USER_INDEX;
    }

    @GetMapping("/user/history/{id}")
    public String getHistories(@PathVariable Long id, Model model) {
        model.addAttribute(HISTORIES, userService.getHistoryById(id));
        model.addAttribute(HEADER, "uživatele");
        return HISTORY_TEMPLATE;
    }

    @GetMapping("/user/deactivate")
    public String deactivateUser(Model model) {
        model.addAttribute(USERS, userService.getAllAllowed());
        return DEACTIVATE_TEMPLATE;
    }

    @PostMapping("/user/deactivate/{id}")
    public String deactivate(@PathVariable Long id, RedirectAttributes redirectAttributes) {
        userService.deactivate(id);
        redirectAttributes.addFlashAttribute(SUCCESS_MESSAGE, "Uživatel byl úspěšně zablokován.");
        return REDIRECT_TO_DEACTIVATE;
    }
}
