package cz.simecek.inspectionmanager.service.category;

import cz.simecek.inspectionmanager.dto.CategoryDTO;
import cz.simecek.inspectionmanager.dto.MachineDTO;
import cz.simecek.inspectionmanager.exception.PropertyNotFoundException;
import cz.simecek.inspectionmanager.exception.UnsafeDeleteException;
import cz.simecek.inspectionmanager.model.Category;
import cz.simecek.inspectionmanager.model.Machine;
import cz.simecek.inspectionmanager.repository.CategoryRepository;
import cz.simecek.inspectionmanager.service.machine.MachineService;
import cz.simecek.inspectionmanager.service.user.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;


@Service
public class CategoryServiceImpl implements CategoryService {

    private final CategoryRepository categoryRepository;
    private final UserService userService;
    private final MachineService machineService;

    @Autowired
    public CategoryServiceImpl(CategoryRepository categoryRepository, UserService userService, MachineService machineService) {
        this.categoryRepository = categoryRepository;
        this.userService = userService;
        this.machineService = machineService;
    }

    private CategoryDTO convertToCategoryDTO(Category category) {
        CategoryDTO categoryDTO = new CategoryDTO();
        categoryDTO.setId(category.getId());
        categoryDTO.setName(category.getName());
        categoryDTO.setCreatedAt(category.getCreatedAt());
        category.setCreatedBy(category.getCreatedBy());
        categoryDTO.setCanBeDeleted(canBeDeleted(category));
        return categoryDTO;
    }

    private Category convertToCategory(CategoryDTO categoryDTO) {
        Category category = new Category();
        category.setName(categoryDTO.getName());
        category.setCreatedAt(categoryDTO.getCreatedAt());
        category.setCreatedBy(category.getCreatedBy());
        return category;
    }

    private Category getCategory(Long categoryId) {
        return categoryRepository.findById(categoryId)
                .orElseThrow(() -> new PropertyNotFoundException("category id: " + categoryId));
    }

    private boolean canBeDeleted(Category category) {
        return category.getMachines().isEmpty();
    }

    public List<CategoryDTO> getAll() {
        return ((List<Category>) categoryRepository.findAll()).stream().map(this::convertToCategoryDTO)
                .collect(Collectors.toList());
    }

    public void save(CategoryDTO categoryDTO) {
        Category category = convertToCategory(categoryDTO);
        category.setCreatedBy(userService.getCurrentUser());
        categoryRepository.save(category);
    }

    public CategoryDTO getById(Long categoryId) {
        Category category = this.getCategory(categoryId);
        return convertToCategoryDTO(category);
    }

    public void delete(Long categoryId) {
        Category category = this.getCategory(categoryId);
        if (canBeDeleted(category)) {
            categoryRepository.delete(category);
        } else {
            throw new UnsafeDeleteException("Category id: " + categoryId);
        }
    }

    public void update(CategoryDTO categoryDTO, Long categoryId) {
        Category category = this.getCategory(categoryId);
        category.setName(categoryDTO.getName());
        categoryRepository.save(category);
    }

    @Override
    public void saveCategoryToMachine(Long categoryId, Long machineId) {
        Category category = getCategory(categoryId);
        Machine machine = machineService.getMachineById(machineId);
        machine.getCategories().add(category);
        machineService.saveMachine(machine);
    }

    public List<MachineDTO> getAllMachinesExcludeByCategoryId(Long id) {
        return machineService.getAllMachinesExcludeByCategoryId(id);
    }
}

