package cz.simecek.inspectionmanager.service.customer;

import cz.simecek.inspectionmanager.dto.CustomerDTO;
import cz.simecek.inspectionmanager.exception.PropertyNotFoundException;
import cz.simecek.inspectionmanager.exception.UnsafeDeleteException;
import cz.simecek.inspectionmanager.model.Customer;
import cz.simecek.inspectionmanager.model.CustomerHistory;
import cz.simecek.inspectionmanager.model.SystemUser;
import cz.simecek.inspectionmanager.repository.CustomerHistoryRepository;
import cz.simecek.inspectionmanager.repository.CustomerRepository;
import cz.simecek.inspectionmanager.service.user.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class CustomerServiceImpl implements CustomerService {

    private final CustomerRepository customerRepository;
    private final CustomerHistoryRepository customerHistoryRepository;
    private final UserService userService;

    @Autowired
    public CustomerServiceImpl(CustomerRepository customerRepository, CustomerHistoryRepository customerHistoryRepository, UserService userService) {
        this.customerRepository = customerRepository;
        this.customerHistoryRepository = customerHistoryRepository;
        this.userService = userService;
    }

    private CustomerDTO convertToCustomerDTO(Customer customer) {
        CustomerDTO customerDTO = new CustomerDTO();
        customerDTO.setId(customer.getId());
        customerDTO.setFirstName(customer.getFirstName());
        customerDTO.setLastName(customer.getLastName());
        customerDTO.setCity(customer.getCity());
        customerDTO.setPhone(customer.getPhone());
        customerDTO.setEmail(customer.getEmail());
        customerDTO.setFocus(customer.getFocus());
        customerDTO.setCreatedAt(customer.getCreatedAt());
        customerDTO.setCanBeDeleted(canBeDeleted(customer));
        return customerDTO;
    }

    private Customer convertToCustomer(CustomerDTO customerDTO) {
        Customer customer = new Customer();
        customer.setId(customerDTO.getId());
        customer.setMachines(customerDTO.getMachines());
        customer.setFirstName(customerDTO.getFirstName());
        customer.setLastName(customerDTO.getLastName());
        customer.setCity(customerDTO.getCity());
        customer.setCity(customerDTO.getCity());
        customer.setPhone(customerDTO.getPhone());
        customer.setEmail(customerDTO.getEmail());
        customer.setFocus(customerDTO.getFocus());
        customer.setCreatedAt(customerDTO.getCreatedAt());
        return customer;
    }

    private Customer updateCustomer(Customer customer, CustomerDTO customerDTO) {
        customer.setFirstName(customerDTO.getFirstName());
        customer.setLastName(customerDTO.getLastName());
        customer.setCity(customerDTO.getCity());
        customer.setPhone(customerDTO.getPhone());
        customer.setEmail(customerDTO.getEmail());
        customer.setFocus(customerDTO.getFocus());
        return customer;
    }

    private boolean canBeDeleted(Customer customer) {
        return customer.getMachines().isEmpty();
    }

    private void saveActionToHistory(String change, Customer customer) {
        SystemUser user = userService.getCurrentUser();
        CustomerHistory history = new CustomerHistory();
        history.setCustomer(customer);
        history.setCreatedAt(new Date(System.currentTimeMillis()));
        history.setCreatedBy(user);
        history.setChange(change);
        customerHistoryRepository.save(history);
    }

    public Customer getCustomer(Long customerId) {
        Customer customer = customerRepository.findByIdAndIsDeleted(customerId, false)
                .orElseThrow(() -> new PropertyNotFoundException("Zákazník id: " + customerId));
        if (customer.getDeleted()) {
            throw new PropertyNotFoundException("Zákazník id: " + customerId);
        }
        return customer;
    }

    public List<CustomerDTO> getAll() {
        return ((List<Customer>) customerRepository.findByIsDeleted(false)).stream()
                .map(this::convertToCustomerDTO).collect(Collectors.toList());
    }

    public void save(CustomerDTO customerDTO) {
        Customer customer = convertToCustomer(customerDTO);
        customer.setCreatedAt(new Date(System.currentTimeMillis()));
        customer.setCreatedBy(userService.getCurrentUser());
        customer.setDeleted(false);
        Customer newCustomer = customerRepository.save(customer);
        saveActionToHistory("Vytvořen nový zákazník: " + newCustomer.toString(), newCustomer);
    }

    public CustomerDTO getById(Long customerId) {
        Customer customer = this.getCustomer(customerId);
        return convertToCustomerDTO(customer);
    }

    public void update(CustomerDTO customerDTO, Long customerId) {
        Customer customer = this.getCustomer(customerId);
        Customer updatedCustomer = updateCustomer(customer, customerDTO);
        Customer newCustomer = customerRepository.save(updatedCustomer);
        saveActionToHistory("Aktualizován zákazník: " + newCustomer.toString(), newCustomer);
    }

    public void delete(Long customerId) {
        Customer customer = this.getCustomer(customerId);
        if (canBeDeleted(customer)) {
            customer.setDeleted(true);
            customerRepository.save(customer);
            saveActionToHistory("Smazán zákazník: " + customer.toString(), customer);
        } else {
            throw new UnsafeDeleteException("Zákazník id: " + customerId);
        }
    }

    @Override
    public List<CustomerHistory> getHistory(Long id) {
        return customerHistoryRepository.findByCustomer_Id(id);
    }

    public List<CustomerDTO> getDeleted() {
        return ((List<Customer>) customerRepository.findByIsDeleted(true)).stream()
                .map(this::convertToCustomerDTO).collect(Collectors.toList());
    }

    public void renew(Long customerId) {
        Customer customer = customerRepository.findByIdAndIsDeleted(customerId, true).orElseThrow(() -> new PropertyNotFoundException("Zákazník id: " + customerId));
        customer.setDeleted(false);
        Customer created = customerRepository.save(customer);
        saveActionToHistory("Obnoven zákazník: " + created.toString(), created);
    }
}
