package cz.simecek.inspectionmanager.service.inspection;

import cz.simecek.inspectionmanager.dto.InspectionDTO;
import cz.simecek.inspectionmanager.exception.PropertyNotFoundException;
import cz.simecek.inspectionmanager.model.*;
import cz.simecek.inspectionmanager.repository.InspectionHistoryRepository;
import cz.simecek.inspectionmanager.repository.InspectionRepository;
import cz.simecek.inspectionmanager.service.machine.MachineServiceImpl;
import cz.simecek.inspectionmanager.service.user.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class InspectionServiceImpl implements InspectionService {

    private final InspectionRepository inspectionRepository;
    private final MachineServiceImpl machineService;
    private final InspectionHistoryRepository inspectionHistoryRepository;
    private final UserService userService;

    @Autowired
    public InspectionServiceImpl(InspectionRepository inspectionRepository, MachineServiceImpl machineService, InspectionHistoryRepository inspectionHistoryRepository, UserService userService) {
        this.inspectionRepository = inspectionRepository;
        this.machineService = machineService;
        this.inspectionHistoryRepository = inspectionHistoryRepository;
        this.userService = userService;
    }

    private InspectionDTO convertToInspectionDTO(Inspection inspection) {
        InspectionDTO inspectionDTO = new InspectionDTO();
        inspectionDTO.setId(inspection.getId());
        inspectionDTO.setMachine(inspection.getMachine());
        inspectionDTO.setDoneAt(inspection.getDoneAt());
        inspectionDTO.setResult(inspection.getResult());
        inspectionDTO.setDescription(inspection.getDescription());
        inspectionDTO.setBill(inspection.getBill());
        inspectionDTO.setPrice(inspection.getPrice());
        inspectionDTO.setCreatedAt(inspection.getCreatedAt());
        return inspectionDTO;
    }

    private Inspection convertToInspection(InspectionDTO inspectionDTO) {
        Inspection inspection = new Inspection();
        inspection.setMachine(inspectionDTO.getMachine());
        inspection.setCreatedAt(inspectionDTO.getCreatedAt());
        return updateInspection(inspection, inspectionDTO);
    }

    private Inspection updateInspection(Inspection inspection, InspectionDTO inspectionDTO) {
        inspection.setDoneAt(inspectionDTO.getDoneAt());
        inspection.setResult(inspectionDTO.getResult());
        inspection.setDescription(inspectionDTO.getDescription());
        inspection.setBill(inspectionDTO.getBill());
        inspection.setPrice(inspectionDTO.getPrice());
        return inspection;
    }

    private Machine getMachine(Long machineId) {
        return machineService.getMachineById(machineId);
    }

    private Inspection getInspection(Long inspectionId) {
        return inspectionRepository.findByIdAndIsDeleted(inspectionId, false)
                .orElseThrow(() -> new PropertyNotFoundException("inspection id: " + inspectionId));
    }

    private void saveActionToHistory(String change, Inspection inspection) {
        SystemUser user = userService.getCurrentUser();
        InspectionHistory history = new InspectionHistory();
        history.setInspection(inspection);
        history.setCreatedAt(new Date(System.currentTimeMillis()));
        history.setCreatedBy(user);
        history.setChange(change);
        inspectionHistoryRepository.save(history);
    }

    public List<InspectionDTO> getAll() {
        return ((List<Inspection>) inspectionRepository.findByIsDeleted(false)).stream().map(this::convertToInspectionDTO)
                .collect(Collectors.toList());
    }

    public List<InspectionDTO> getByMachine(Long machineId) {
        Machine machine = this.getMachine(machineId);
        return ((List<Inspection>) inspectionRepository.findByIsDeletedAndMachine_Id(false, machineId))
                .stream().map(this::convertToInspectionDTO).collect(Collectors.toList());
    }

    public void save(InspectionDTO inspectionDTO, Long machineId) {
        Machine machine = this.getMachine(machineId);
        inspectionDTO.setCreatedAt(new Date(System.currentTimeMillis()));
        Inspection inspection = convertToInspection(inspectionDTO);
        inspection.setMachine(machine);
        inspection.setCreatedBy(userService.getCurrentUser());
        inspection.setDeleted(false);
        Inspection createdInspection = inspectionRepository.save(inspection);
        saveActionToHistory("Vytvořena nová kontrola :" + createdInspection.toString(), createdInspection);
    }

    public InspectionDTO getById(Long inspectionId) {
        Inspection inspection = this.getInspection(inspectionId);
        return convertToInspectionDTO(inspection);
    }

    public void update(InspectionDTO inspectionDTO, Long inspectionId) {
        Inspection inspection = this.getInspection(inspectionId);
        Inspection updatedInspection = updateInspection(inspection, inspectionDTO);
        Inspection createdInspection = inspectionRepository.save(updatedInspection);
        saveActionToHistory("Aktualizována kontrola :" + createdInspection.toString(), createdInspection);
    }

    public void delete(Long inspectionId) {
        Inspection inspection = this.getInspection(inspectionId);
        inspection.setDeleted(true);
        Inspection createdInspection = inspectionRepository.save(inspection);
        saveActionToHistory("Smazána kontrola :" + createdInspection.toString(), createdInspection);
    }

    @Override
    public List<InspectionHistory> getHistoryById(Long id) {
        return inspectionHistoryRepository.findByInspection_Id(id);
    }

    @Override
    public List<InspectionDTO> getDeleted() {
        return ((List<Inspection>) inspectionRepository.findByIsDeleted(true)).stream().map(this::convertToInspectionDTO)
                .collect(Collectors.toList());
    }

    @Override
    public void renew(Long id) {
        Inspection inspection = inspectionRepository.findByIdAndIsDeleted(id, true)
                .orElseThrow(() -> new PropertyNotFoundException("Kontrola id: " + id));
        inspection.setDeleted(false);
        Inspection created = inspectionRepository.save(inspection);
        saveActionToHistory("Obnovena kontrola: " + created.toString(), created);
    }
}
