package cz.simecek.inspectionmanager.service.machine;

import cz.simecek.inspectionmanager.dto.MachineDTO;
import cz.simecek.inspectionmanager.exception.PropertyNotFoundException;
import cz.simecek.inspectionmanager.exception.UnsafeDeleteException;
import cz.simecek.inspectionmanager.model.*;
import cz.simecek.inspectionmanager.repository.MachineHistoryRepository;
import cz.simecek.inspectionmanager.repository.MachineRepository;
import cz.simecek.inspectionmanager.service.customer.CustomerServiceImpl;
import cz.simecek.inspectionmanager.service.user.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class MachineServiceImpl implements MachineService {

    private final MachineRepository machineRepository;
    private final CustomerServiceImpl customerService;
    private final MachineHistoryRepository machineHistoryRepository;
    private final UserService userService;

    @Autowired
    public MachineServiceImpl(MachineRepository machineRepository, CustomerServiceImpl customerService, MachineHistoryRepository machineHistoryRepository, UserService userService) {
        this.machineRepository = machineRepository;
        this.customerService = customerService;
        this.machineHistoryRepository = machineHistoryRepository;
        this.userService = userService;
    }

    private MachineDTO convertToMachineDTO(Machine machine) {
        MachineDTO machineDTO = new MachineDTO();
        machineDTO.setId(machine.getId());
        machineDTO.setCustomer(machine.getCustomer());
        machineDTO.setInspections(machine.getInspections());
        machineDTO.setCategories(machine.getCategories());
        machineDTO.setName(machine.getName());
        machineDTO.setCode(machine.getCode());
        machineDTO.setNextInspectionAt(machine.getNextInspectionAt());
        machineDTO.setNote(machine.getNote());
        machineDTO.setIsBoughtLocally(machine.getIsBoughtLocally());
        machineDTO.setBoughtAt(machine.getBoughtAt());
        machineDTO.setCreatedAt(machine.getCreatedAt());
        machineDTO.setIsWorking(machine.getIsWorking());
        machineDTO.setCanBeDeleted(canBeDeleted(machine));
        return machineDTO;
    }

    private Machine convertToMachine(MachineDTO machineDTO) {
        Machine machine = new Machine();
        machine.setCustomer(machineDTO.getCustomer());
        machine.setInspections(machineDTO.getInspections());
        machine.setCategories(machine.getCategories());
        return updateMachine(machine, machineDTO);
    }

    private Machine updateMachine(Machine machine, MachineDTO machineDTO) {
        machine.setName(machineDTO.getName());
        machine.setCode(machineDTO.getCode());
        machine.setNextInspectionAt(machineDTO.getNextInspectionAt());
        machine.setNote(machineDTO.getNote());
        machine.setIsBoughtLocally(machineDTO.getIsBoughtLocally());
        machine.setBoughtAt(machineDTO.getBoughtAt());
        machine.setIsWorking(machineDTO.getIsWorking());
        return machine;
    }

    private boolean canBeDeleted(Machine machine) {
        return machine.getInspections().isEmpty();
    }

    private void saveActionToHistory(String change, Machine machine) {
        SystemUser user = userService.getCurrentUser();
        MachineHistory history = new MachineHistory();
        history.setMachine(machine);
        history.setCreatedAt(new Date(System.currentTimeMillis()));
        history.setCreatedBy(user);
        history.setChange(change);
        machineHistoryRepository.save(history);
    }

    public Machine getMachineById(Long machineId) {
        return machineRepository.findByIdAndIsDeleted(machineId, false)
                .orElseThrow(() -> new PropertyNotFoundException("Stroj id: " + machineId));
    }

    @Override
    public void saveMachine(Machine machine) {
        Machine createdMachine = machineRepository.save(machine);
        saveActionToHistory("Vytvořen nový stroj: " + createdMachine.toString(), createdMachine);
    }

    private Customer getCustomer(Long customerId) {
        return customerService.getCustomer(customerId);
    }

    public List<MachineDTO> getAll() {
        return ((List<Machine>) machineRepository.findByIsDeleted(false)).stream().map(this::convertToMachineDTO)
                .collect(Collectors.toList());
    }

    public void save(MachineDTO machineDTO, Long customerId) {
        Customer customer = this.getCustomer(customerId);
        Machine machine = convertToMachine(machineDTO);
        machine.setCreatedAt(new Date(System.currentTimeMillis()));
        machine.setCreatedBy(userService.getCurrentUser());
        machine.setCustomer(customer);
        machine.setDeleted(false);
        Machine createdMachine = machineRepository.save(machine);
        saveActionToHistory("Vytvořen nový stroj: " + createdMachine.toString(), createdMachine);
    }

    public List<MachineDTO> getByCustomer(Long customerId) {
        return (machineRepository.findByIsDeletedAndCustomer_Id(false, customerId)).stream().map(this::convertToMachineDTO)
                .collect(Collectors.toList());
    }

    public MachineDTO getById(Long machineId) {
        Machine machine = this.getMachineById(machineId);
        return convertToMachineDTO(machine);
    }

    public void update(MachineDTO machineDTO, Long machineId) {
        Machine machine = this.getMachineById(machineId);
        Machine updatedMachine = updateMachine(machine, machineDTO);
        Machine cratedMachine = machineRepository.save(updatedMachine);
        saveActionToHistory("Aktualizován stroj: " + cratedMachine.toString(), cratedMachine);
    }

    public void delete(Long machineId) {
        Machine machine = this.getMachineById(machineId);
        if (canBeDeleted(machine)) {
            machine.setDeleted(true);
            machineRepository.save(machine);
            saveActionToHistory("Smazán stroj: " + machine.toString(), machine);
        } else {
            throw new UnsafeDeleteException("Stroj id: " + machineId);
        }
    }

    public List<MachineDTO> findLateInspections() {
        Date now = new Date(System.currentTimeMillis());
        return (machineRepository.findLateInspection(now)).stream().map(this::convertToMachineDTO)
                .collect(Collectors.toList());
    }

    public List<MachineHistory> getHistoryById(Long id) {
        return machineHistoryRepository.findByMachine_Id(id);
    }

    @Override
    public List<MachineDTO> getDeleted() {
        return ((List<Machine>) machineRepository.findByIsDeleted(true)).stream().map(this::convertToMachineDTO)
                .collect(Collectors.toList());
    }

    @Override
    public void renew(Long id) {
        Machine machine = machineRepository.findByIdAndIsDeleted(id, true)
                .orElseThrow(() -> new PropertyNotFoundException("Stroj id: " + id));
        machine.setDeleted(false);
        Machine created = machineRepository.save(machine);
        saveActionToHistory("Obnoven stroj: " + created.toString(), created);
    }

    @Override
    public List<MachineDTO> getAllMachinesExcludeByCategoryId(Long id) {
        return ((List<Machine>) machineRepository.findByIsDeletedAndCategories_IdNot(false, id)).stream().map(this::convertToMachineDTO)
                .collect(Collectors.toList());
    }

    @Override
    public List<MachineDTO> getAllByCategory(Long id) {
        return ((List<Machine>) machineRepository.findByIsDeletedAndCategories_Id(false, id)).stream().map(this::convertToMachineDTO)
                .collect(Collectors.toList());
    }
}
