package cz.simecek.inspectionmanager.service.spare_part;

import cz.simecek.inspectionmanager.dto.SparePartDTO;
import cz.simecek.inspectionmanager.exception.PropertyNotFoundException;
import cz.simecek.inspectionmanager.model.*;
import cz.simecek.inspectionmanager.repository.SparePartHistoryRepository;
import cz.simecek.inspectionmanager.repository.SparePartRepository;
import cz.simecek.inspectionmanager.service.machine.MachineService;
import cz.simecek.inspectionmanager.service.user.UserService;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class SparePartServiceImpl implements SparePartService {

    private final SparePartRepository sparePartRepository;
    private final MachineService machineService;
    private final UserService userService;
    private final SparePartHistoryRepository sparePartHistoryRepository;

    public SparePartServiceImpl(SparePartRepository sparePartRepository, MachineService machineService, UserService userService, SparePartHistoryRepository sparePartHistoryRepository) {
        this.sparePartRepository = sparePartRepository;
        this.machineService = machineService;
        this.userService = userService;
        this.sparePartHistoryRepository = sparePartHistoryRepository;
    }

    private SparePartDTO convertToDTO(SparePart sparePart) {
        SparePartDTO sparePartDTO = new SparePartDTO();
        sparePartDTO.setId(sparePart.getId());
        sparePartDTO.setCode(sparePart.getCode());
        sparePartDTO.setName(sparePart.getName());
        sparePartDTO.setPrice(sparePart.getPrice());
        sparePartDTO.setMachines(sparePart.getMachines());
        return sparePartDTO;
    }

    private SparePart convertToPart(SparePartDTO sparePartDTO) {
        SparePart sparePart = new SparePart();
        sparePart.setId(sparePartDTO.getId());
        return updatePartByDTO(sparePart, sparePartDTO);
    }

    private SparePart updatePartByDTO(SparePart sparePart, SparePartDTO dto) {
        sparePart.setName(dto.getName());
        sparePart.setCode(dto.getCode());
        sparePart.setPrice(dto.getPrice());
        sparePart.setMachines(dto.getMachines());
        return sparePart;
    }

    private SparePart getPartById(Long id) {
        return sparePartRepository.findByIdAndIsDeleted(id, false).orElseThrow(() -> new PropertyNotFoundException("Náhradní díl id: " + id));
    }

    private Machine getMachineById(Long id) {
        return machineService.getMachineById(id);
    }

    private void saveActionToHistory(String change, SparePart sparePart) {
        SystemUser user = userService.getCurrentUser();
        SparePartHistory history = new SparePartHistory();
        history.setSparePart(sparePart);
        history.setCreatedAt(new Date(System.currentTimeMillis()));
        history.setCreatedBy(user);
        history.setChange(change);
        sparePartHistoryRepository.save(history);
    }

    @Override
    public List<SparePartDTO> getAll() {
        return ((List<SparePart>) sparePartRepository.findByIsDeleted(false)).stream().map(this::convertToDTO).collect(Collectors.toList());
    }

    @Override
    public void save(SparePartDTO partDTO, Long machineId) {
        Machine machine = getMachineById(machineId);
        SparePart sparePart = convertToPart(partDTO);
        sparePart.setDeleted(false);
        machine.getSpareParts().add(sparePart);
        machineService.saveMachine(machine);
        SparePart created = sparePartRepository.findByCode(sparePart.getCode()).orElseThrow(() -> new PropertyNotFoundException("Nepodařilo se uložit náhradní díl"));
        saveActionToHistory("Vytvoření nového náhradního dílu: " + sparePart.toString(), created);
    }

    @Override
    public SparePartDTO getById(Long partId) {
        return convertToDTO(getPartById(partId));
    }

    @Override
    public void delete(Long partId) {
        SparePart sparePart = getPartById(partId);
        sparePart.setDeleted(true);
        sparePartRepository.save(sparePart);
        saveActionToHistory("Smazání náhradního dílu: " + sparePart.toString(), sparePart);
    }

    @Override
    public void update(SparePartDTO partDTO, Long partId) {
        SparePart part = updatePartByDTO(getPartById(partId), partDTO);
        SparePart updated = sparePartRepository.save(part);
        saveActionToHistory("Aktualizace náhradního dílu: " + updated.toString(), updated);
    }

    @Override
    public List<SparePartDTO> getByMachineId(Long id) {
        return ((List<SparePart>) sparePartRepository.findByIsDeletedAndMachines_Id(false, id)).stream().map(this::convertToDTO).collect(Collectors.toList());
    }

    @Override
    public List<SparePartHistory> getHistoryById(Long id) {
        return sparePartHistoryRepository.findBySparePart_Id(id);
    }

    @Override
    public List<SparePartDTO> getDeleted() {
        return ((List<SparePart>) sparePartRepository.findByIsDeleted(true)).stream().map(this::convertToDTO)
                .collect(Collectors.toList());
    }

    @Override
    public void renew(Long id) {
        SparePart sparePart = sparePartRepository.findByIdAndIsDeleted(id, true)
                .orElseThrow(() -> new PropertyNotFoundException("Náhradní díl id: " + id));
        sparePart.setDeleted(false);
        SparePart created = sparePartRepository.save(sparePart);
        saveActionToHistory("Obnoven náhradní díl: " + created.toString(), created);
    }
}

