package cz.simecek.inspectionmanager.service.user;

import cz.simecek.inspectionmanager.dto.RoleDTO;
import cz.simecek.inspectionmanager.dto.UserPasswordDTO;
import cz.simecek.inspectionmanager.exception.InvalidPasswordChangeException;
import cz.simecek.inspectionmanager.exception.PropertyNotFoundException;
import cz.simecek.inspectionmanager.dto.PasswordDTO;
import cz.simecek.inspectionmanager.model.SystemUser;
import cz.simecek.inspectionmanager.model.SystemUserHistory;
import cz.simecek.inspectionmanager.repository.SystemUserHistoryRepository;
import cz.simecek.inspectionmanager.repository.SystemUserRepository;
import cz.simecek.inspectionmanager.dto.UserDTO;
import cz.simecek.inspectionmanager.service.role.RoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.util.List;
import java.util.stream.Collectors;


@Service
public class UserServiceImpl implements UserService {

    private final SystemUserRepository userRepository;
    private final RoleService roleService;
    private final BCryptPasswordEncoder bCryptPasswordEncoder;
    private final SystemUserHistoryRepository systemUserHistoryRepository;

    @Autowired
    public UserServiceImpl(SystemUserRepository userRepository, RoleService roleService, BCryptPasswordEncoder bCryptPasswordEncoder, SystemUserHistoryRepository systemUserHistoryRepository) {
        this.userRepository = userRepository;
        this.roleService = roleService;
        this.bCryptPasswordEncoder = bCryptPasswordEncoder;
        this.systemUserHistoryRepository = systemUserHistoryRepository;
    }

    private UserDTO convertToUserDTO(SystemUser user) {
        UserDTO userDTO = new UserDTO();
        userDTO.setId(user.getId());
        userDTO.setUsername(user.getUserName());
        userDTO.setRole(user.getRole());
        userDTO.setBlocked(user.getBlocked());
        return userDTO;
    }

    private SystemUser getUserById(Long id) {
        return userRepository.findById(id).orElseThrow(() -> new PropertyNotFoundException("Uživatel id: " + id));
    }

    private void saveActionToHistory(String change, SystemUser systemUser) {
        SystemUserHistory history = new SystemUserHistory();
        if (getCurrentUser() != null) {
            history.setCreatedBy(getCurrentUser());
        }
        history.setSystemUser(systemUser);
        history.setCreatedAt(new Date(System.currentTimeMillis()));
        history.setChange(change);
        systemUserHistoryRepository.save(history);
    }

    @Override
    public void save(UserDTO userdto) {
        SystemUser user = new SystemUser();
        user.setUserName(userdto.getUsername());
        user.setPassword(bCryptPasswordEncoder.encode(userdto.getPassword()));
        user.setRole(roleService.getById(userdto.getRole().getId()));
        user.setBlocked(userdto.getBlocked());
        SystemUser created = userRepository.save(user);
        saveActionToHistory("Vytvořen uživatel: " + created.toString(), created);
    }

    @Override
    public SystemUser findByUsername(String username) {
        return userRepository.findByUserName(username);
    }

    @Override
    public void delete(Long id) {
        SystemUser user = getUserById(id);
        userRepository.delete(user);
    }

    @Override
    public List<UserDTO> getAll() {
        return userRepository.findAll().stream().map(this::convertToUserDTO).collect(Collectors.toList());
    }

    @Override
    public UserDTO findById(Long id) {
        SystemUser user = getUserById(id);
        return convertToUserDTO(user);
    }

    @Override
    public void changePassword(PasswordDTO passwordDTO) {
        if (!passwordDTO.getNewPassword().equals(passwordDTO.getConfirmNewPassword())) {
            throw new InvalidPasswordChangeException();
        }
        SystemUser user = getUserById(passwordDTO.getUserId());
        user.setPassword(bCryptPasswordEncoder.encode(passwordDTO.getNewPassword()));
        SystemUser created = userRepository.save(user);
        saveActionToHistory("Aktualizováno uživatelské heslo administrátorem: " + created.toString(), created);
    }

    public void changeRole(RoleDTO roleDTO) {
        SystemUser user = getUserById(roleDTO.getUserId());
        user.setRole(roleDTO.getNewRole());
        SystemUser created = userRepository.save(user);
        saveActionToHistory("Aktualizována uživatelská role: " + created.toString(), created);

    }

    public SystemUser getCurrentUser() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        if (authentication instanceof AnonymousAuthenticationToken) {
            return null;
        }
        UserDetails userDetails = (UserDetails) authentication.getPrincipal();
        String username = userDetails.getUsername();
        return userRepository.findByUserName(username);
    }

    @Override
    public List<SystemUser> getAllBlocked() {
        return userRepository.findByIsBlocked(true);
    }

    @Override
    public void activate(Long id) {
        SystemUser user = getUserById(id);
        user.setBlocked(false);
        SystemUser created = userRepository.save(user);
        saveActionToHistory("Aktivován uživatel: " + created.toString(), created);
    }

    @Override
    public List<SystemUserHistory> getHistoryById(Long id) {
        return systemUserHistoryRepository.findBySystemUser_Id(id);
    }

    @Override
    public List<SystemUser> getAllAllowed() {
        return userRepository.findByIsBlocked(false);
    }

    @Override
    public void deactivate(Long id) {
        SystemUser user = getUserById(id);
        user.setBlocked(true);
        SystemUser created = userRepository.save(user);
        saveActionToHistory("Deaktivován uživatel: " + created.toString(), created);
    }

    @Override
    public void updatePasswordByUser(UserPasswordDTO userPasswordDTO) {
        if (getCurrentUser() != null) {
            SystemUser user = getCurrentUser();
            if (validateUserPasswordForm(userPasswordDTO, user.getPassword())) {
                user.setPassword(bCryptPasswordEncoder.encode(userPasswordDTO.getNewPassword()));
                SystemUser created = userRepository.save(user);
                saveActionToHistory("Aktualizováno heslo uživatelem: " + created.toString(), created);
            }
        }
    }

    private Boolean validateUserPasswordForm(UserPasswordDTO userPasswordDTO, String currentPassword) {
        if (!userPasswordDTO.getNewPassword().equals(userPasswordDTO.getConfirmNewPassword())) {
            throw new InvalidPasswordChangeException();
        }
        if (!bCryptPasswordEncoder.matches(userPasswordDTO.getOldPassword(), currentPassword)) {
            throw new InvalidPasswordChangeException();
        }
        return true;
    }
}
